#!/usr/bin/env python3
# ./1_generate_promoter_sequences.py /home/jhuang/REFs/gencode.v43.annotation.gtf.db
#improve the code to exact alsogene_type except for ensemble_ids and gene_symbol, and save them in the heads.

# # Create the database from the GTF file
# gtf_file = 'gencode.v43.annotation.gtf'
# db_file = 'gencode.v43.annotation.gtf.db'
# db = gffutils.create_db(gtf_file, dbfn=db_file, force=True, merge_strategy='merge', verbose=True)

import gffutils
from pyfaidx import Fasta
import argparse
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from Bio.Seq import Seq

def extract_promoter_sequences(gencode_db, genome_records, upstream_length=3000, downstream_length=3000):
    db = gffutils.FeatureDB(gencode_db, keep_order=True)
    promoters = []
    for gene in db.features_of_type('gene'): #CDS
        gene_type = gene.attributes.get('gene_type', [''])[0]
        #if gene_type == 'protein_coding':
        if True:
            if gene.strand == '+':
                promoter_start = max(gene.start - upstream_length, 1)
                promoter_end = gene.start + downstream_length - 1
            else: # gene.strand == '-'
                promoter_start = gene.end - downstream_length + 1
                promoter_end = min(gene.end + upstream_length, len(genome_records[gene.chrom]))
            promoter_seq = str(genome_records[gene.chrom][promoter_start-1:promoter_end])
            if gene.strand == '-':
                promoter_seq = str(Seq(promoter_seq).reverse_complement())
            
            gene_symbol = gene.attributes.get('gene_name', [''])[0]
            gene_type = gene.attributes.get('gene_type', [''])[0]
            
            #20042 in which gene_type == 'protein_coding', while 62703 all types.
            header = f"{gene.id} | Symbol: {gene_symbol} | Type: {gene_type}"
            #header = f"{gene_symbol}"
            #record = SeqRecord(Seq(promoter_seq.upper()), id=gene.id, description=gene_symbol)
            record = SeqRecord(Seq(promoter_seq.upper()), id=gene.id, description=header)
            promoters.append(record)
    
    return promoters

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Plot peak distribution around TSS.')
    parser.add_argument('gencode_db', type=str, help='Path to the gencode.v43.annotation.gtf.db file')
    args = parser.parse_args()
    
    genome_records = Fasta('/ref/Homo_sapiens/UCSC/hg38/Sequence/WholeGenomeFasta/genome.fa')
    promoter_sequences = extract_promoter_sequences(args.gencode_db, genome_records)
    SeqIO.write(promoter_sequences, "promoter_sequences_3000_3000.fasta", "fasta")
